import { fireEvent, render, screen } from '@testing-library/react';
import { Navigator } from '../Components/Navigator/index';

describe('Generic navigator tests', () => {
    const mockFirebase = jest.fn();
    const listElement = (props: {data: string}) => { return <p>{props.data}</p>};

    beforeEach(() => {
        mockFirebase.mockImplementation((param: string) => {
            switch (param) {
                case 'param1':
                    return Promise.resolve(['test1', 'test2']);
                case 'param2':
                    return Promise.resolve(['test3']);
                case 'param3':
                    return Promise.resolve([]);
            }
        });
    });

    test('Should display the title correctly', async () => {
        
        const mockHandler = jest.fn();
    
        render(<Navigator<string> getter={mockFirebase} title={'test_title'} param={'param1'} handler={mockHandler} listElement={listElement}/>);
    
        const titleMatches = await screen.findAllByText('test_title');

        expect(titleMatches).toHaveLength(1);
    });

    test('Complex element from the list should have the chosen html pattern', async () => {
        const mockHandler = jest.fn();
        const complexMockFirebase = jest.fn();
        complexMockFirebase.mockImplementation(() => (Promise.resolve([
            {
                id: 'id1',
                name: 'name1',
                price: 3,
            }
        ])));
        const complexListElement = (props: {data: {name: string, id: string, price: number}}) => 
            <div data-testid={props.data.id}>
                <p>Id : {props.data.id}</p>
                <p>Name: {props.data.name}</p>
                <p>Price: {props.data.price}</p>
            </div>

        render(<Navigator<{name: string, id: string, price: number}> getter={complexMockFirebase} title={'test'} param={''} handler={mockHandler} listElement={complexListElement}/>);
        
        const foundElement = await screen.findAllByTestId('id1');
        expect(foundElement).toHaveLength(1);
        expect(foundElement[0]).toContainHTML('<div data-testid="id1"><p>Id : id1</p><p>Name: name1</p><p>Price: 3</p></div>')
    });

    test('Multiple elements queried from Firebase should display correctly', async () => {
        
        const mockHandler = jest.fn();
        const listElement = (props: {data: string}) => { return <p>test_item : {props.data}</p>}
    
        render(<Navigator<string> getter={mockFirebase} title={'test'} param={'param1'} handler={mockHandler} listElement={listElement}/>);
        
        const elts = await screen.findAllByText(/test_item/);
        expect(elts).toHaveLength(2);
    });

    test('If there is no item on Firebase, nothing should be in the list', async () => {
        
        const mockHandler = jest.fn();
        const listElement = (props: {data: string}) => { return <p>test_item : {props.data}</p>}
        
        await render(<Navigator<string> getter={mockFirebase} title={'test'} param={'param3'} handler={mockHandler} listElement={listElement}/>);
        
        const elt = screen.queryByText(/test_item/);
        expect(elt).toBeNull();
    });

    test('If no list item is clicked, handler should not be called', async () => {
        
        const mockHandler = jest.fn();
    
        render(<Navigator<string> getter={mockFirebase} title={'test'} param={'param1'} handler={mockHandler} listElement={listElement}/>);
    
        expect(mockHandler.mock.calls).toEqual([]);
    });
    
    test('If I click an element of the list, it should be processed by the handler once', async () => {
        
        const mockHandler = jest.fn();
        const listElement = (props: {data: string}) => { return <p>{props.data}</p>}       
    
        render(<Navigator<string> getter={mockFirebase} title={'test'} param={'param1'} handler={mockHandler} listElement={listElement}/>);
        
        const elt1 = await screen.findByText('test1');
        
        fireEvent.click(elt1);
        
        const expectedCalls = [['test1']];
        expect(expectedCalls).toEqual(mockHandler.mock.calls);
    });

    test('Handler should process clicked elements in the correct order', async () => {
        
        const mockHandler = jest.fn();
        const listElement = (props: {data: string}) => { return <p>{props.data}</p>}       
    
        render(<Navigator<string> getter={mockFirebase} title={'test'} param={'param1'} handler={mockHandler} listElement={listElement}/>);
        
        const elt1 = await screen.findByText('test1');
        fireEvent.click(elt1);

        const elt2 = await screen.findByText('test2');
        fireEvent.click(elt2);
        
        const expectedCalls = [['test1'], ['test2']];
        expect(expectedCalls).toEqual(mockHandler.mock.calls);
    });
})

